//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2013
//#
//#
//# File Name:  EPT_570_AP_U2_Top.v
//# Author:     Earth People Technology
//# Date:       June 27, 2013
//# Revision:   A
//#
//# Development: EPT Data Collection Project 
//# Application: Altera MAX II CPLD
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			6/27/13 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_570_AP_U2_Top (

	
	input  wire [1:0]           aa,
	input  wire [1:0]           bc_in,
	output wire [2:0]           bc_out,
	inout  wire [7:0]		    bd_inout,
	
	output wire  [1:0]          LB_SER,       //XIOH -- J10
	output wire  [5:0]          LB_AD,        //AD   -- J9
	input wire   [7:0]          LB_IOH,       //XIOH -- J10
	output wire  [7:0]          LB_IOL,       //XIOL -- J8
	
	
	//Transceiver Control Signals
	output wire                TR_DIR_1,
	output wire                TR_OE_1,
	
	output wire                TR_DIR_2,
	output wire                TR_OE_2,

	output wire                TR_DIR_3,
	output wire                TR_OE_3,

	
	input wire                 SW_USER_1,
	input wire                 SW_USER_2,

	output wire [3:0]          LED
	);

   //-----------------------------------------------
   // Parameters Declarations
   //-----------------------------------------------

   parameter                   GLOBAL_RESET_COUNT = 12'h09c8;
   
   //----------------------------------------------
   // Parameter Declarations                        
   //----------------------------------------------
   parameter IDLE                   = 0,
	     RECEIVE_DATA				= 1,
		 SEND_TO_HOST			    = 2,
		 RECEIVE_COMPLETE			= 3;
		 
   parameter WRITE_REGISTER 		= 1,
		 LOAD_SHIFT_REG				= 2,
		 DELAY_WRITE_DATA_BYTE      = 3,
	     WRITE_DATA_BYTE			= 4,
	     WRITE_DATA_COMPLETE		= 5;
		 
   //----------------------------------------------
   // Bit Count Parameter Declarations                        
   //----------------------------------------------
   parameter BIT_COUNT_IDLE         = 0,
			 BIT_COUNT_CLK_HI		= 1,
			 BIT_COUNT_CLK_LOW		= 2,
			 BIT_COUNT_INC_BYTE		= 3;
			 
   //----------------------------------------------
   // Parameter For SPI ready for next byte counter                       
   //----------------------------------------------
   parameter SPI_READY_COUNT        = 13'h19c8;			 
 	
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
	//Finite State Machine control registers
	reg 	[3:0]              state_in, next_in;
	reg 	[5:0]              state_out, next_out;

	//Trigger Signals
	reg  [7:0]                 trigger_out;
	wire [7:0]                 trigger_in_byte;	
	
	//Transfer registers
	wire                       start_transfer;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	wire  [7:0]                transfer_out_byte;
	wire                       transfer_busy;

    //Transfer Control registers
	wire   [7:0]               control_register;
	wire                       reset_spi_cntrl;
	
 	//Reset signals
	wire                       reset;
	reg [11:0]                 reset_counter;
	reg                        reset_signal_reg;
	
	//SPI In/Out Registers
	wire                       spi_clk;
	wire                       spi_data_in;
	reg                        spi_data_out;
	wire                       spi_cs;

    // Bit Count Registers
	reg [3:0]                  in_bit_count;
	reg [3:0]                  out_bit_count;
	
	// SPI Shift Registers
	reg [7:0]                  shift_in_reg, shift_out_reg;
	reg                        read_spi_data_in_once_flag;
	reg                        write_spi_data_out_once_flag;

	reg                        write_spi_data_out_response_once;
	reg                        spi_clk_reg;
	
	// Tel-D signals to the MSP430
	reg                        spi_ready_for_next_byte;
	reg                        spi_ready_for_next_byte_reg;
	reg [12:0]                 spi_ready_for_next_byte_counter;
	reg                        spi_rf_int; 
	wire [7:0]                 store_spi_output_byte;
	reg [7:0]                  rf_int_store_output_byte;
	reg [7:0]                  response_store_output_byte;
	
	//Response Byte Request from the Host PC
	reg                        response_byte_req;
	reg [3:0]                  response_byte_counter;
		
	// in bit counter
	reg [1:0]                  bit_count_state;
	
`ifdef SIM
   reg [8*26:1] state_name_in;
   reg [8*26:1] state_name_out;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
   // Bus Transceiver Controls for LB_IOH
   assign            TR_DIR_1  = 1'b1; //1 = A to B; 0 = B to A
   assign            TR_OE_1  = 1'b0;  //1 = Off; 0 = On

   // Bus Transceiver Controls for LB_AD and LB_SER
   assign            TR_DIR_2  = 1'b0; //1 = A to B; 0 = B to A
   assign            TR_OE_2  = 1'b0;  //1 = Off; 0 = On
   
   // Bus Transceiver Controls for LB_IOL   
   assign            TR_DIR_3  = 1'b0; //1 = A to B; 0 = B to A
   assign            TR_OE_3  = 1'b0;  //1 = Off; 0 = On

   //Clock and Reset
   assign            CLK_66 = aa[1];
   assign            RST = reset;
   assign            reset = reset_signal_reg;
   
   //Active Transfer EndTerm Registers
   assign            transfer_out_byte = shift_in_reg;
   assign            start_transfer = state_in[SEND_TO_HOST] ? 1'b1 : 1'b0;
   
    //Control Register Map
   assign            reset_spi_cntrl = control_register[1];
   
   //Store SPI Transmit byte to Host
   assign            store_spi_output_byte = spi_rf_int ? rf_int_store_output_byte : response_store_output_byte;

   //Assign the unused signals in LB_IOL to z
   //assign            LB_IOL[7:6] = 2'hz;
 
   //SPI bus stuff
   assign            spi_cs = LB_IOH[1]; 
   assign            spi_clk = LB_IOH[2];
   assign            spi_data_in = LB_IOH[3];
   assign            LB_SER[0] = 1'b0;
   assign            LB_SER[1] = spi_data_out;
   assign            LB_IOL[0] = spi_ready_for_next_byte;
   assign            LB_IOL[1] = spi_rf_int;
   
   //Test Signals
   assign            LB_IOL[2] = store_spi_output_byte[0];
   assign            LB_IOL[3] = store_spi_output_byte[1];//transfer_in_received;//transfer_out_byte[0];//
   assign            LB_IOL[4] = store_spi_output_byte[2];//store_spi_output_byte[0];//
   assign            LB_IOL[5] = store_spi_output_byte[3];//store_spi_output_byte[1];//
   assign            LB_IOL[6] = trigger_in_byte[1];//store_spi_output_byte[2];//
   assign            LB_IOL[7] = spi_data_out; //state_in[1];//trigger_in_byte[0];//state_in[0];
   
   assign            LB_AD[0] = state_in[2];//trigger_in_byte[0];//
   assign            LB_AD[1] = state_in[3];//start_transfer;//
  
   //LED3 is used to signify to the  user that the Start 
   //switch is enabled
   assign            LED[3] =  spi_cs ? 1'b0 : 1'bz;  
   assign            LED[2] = spi_clk ? 1'b0 : 1'bz;
   assign            LED[1] = spi_data_in ? 1'b0 : 1'bz;
   assign            LED[0] = spi_data_out ? 1'b0 : 1'bz;
     
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66 or negedge aa[0] or posedge reset_spi_cntrl)
	begin
	  if(!aa[0])
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else if(reset_spi_cntrl)
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else 
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // Set trigger_out bit 0 high to request a  
   // response byte from the Host.   
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
   begin
      if (!RST)
	  begin
         trigger_out      <= 0;
		 response_byte_req <= 1'b0;
		 response_byte_counter <= 0;
	  end
	  else
	  begin
	     if(state_in[RECEIVE_COMPLETE] & !response_byte_req)
		 begin
			   trigger_out      <= 0;
			   response_byte_req <= 1'b1;
		 end
		 else if(response_byte_req)
		 begin
		     if(response_byte_counter < 4'hf)
			    response_byte_counter <= response_byte_counter + 1'd1;
		     else
			 begin
                trigger_out      <= 8'h1;
                if(state_in[IDLE])	
                begin				
		           response_byte_req <= 1'b0;
				   response_byte_counter <= 0;
				end
			 end
		 end
		 else
             trigger_out      <= 0;
	  end
	end
	
   //-----------------------------------------------
   // RF Interrupt register. Informs the MSP430 that 
   // the Tel-D chip has a byte ready to transfer   
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	begin
	     spi_rf_int <= 1'b0;
		 rf_int_store_output_byte <= 0;
	end
	else
	  begin
	     if(trigger_in_byte[2])
		 begin
		     //store_spi_output_byte <= transfer_in_byte;
		     rf_int_store_output_byte <= transfer_in_byte;
	         spi_rf_int <= 1'b1;
		 end
	     else if(state_in[RECEIVE_COMPLETE])
		 begin
	       spi_rf_int <= 1'b0;
		 end
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //-----------------------------------------------
   // Respsonse Message from the Tel-D to the MSP430. 
   // This is single byte by byte message prepared by
   // the host.   
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	     response_store_output_byte <= 0;
	else
	  begin
	     if(trigger_in_byte[1])
		 begin
		     response_store_output_byte <= transfer_in_byte;
		 end
	  end
     end // always @ (posedge CLK_66 or negedge RST)


   //-----------------------------------------------
   // Tel-D spi_ready_for_next_byte, informs the MSP430
   // to hold off sending next byte until the Tel-D is ready  
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	begin
	    spi_ready_for_next_byte <= 1'b0;
	    spi_ready_for_next_byte_reg <= 1'b0;
		spi_ready_for_next_byte_counter <= 0;
	end
	else
	  begin
	     if(state_in[SEND_TO_HOST] & !spi_ready_for_next_byte_reg)
		 begin
		   spi_ready_for_next_byte_reg <= 1'b1;
	       spi_ready_for_next_byte <= 1'b1;
		 end
	     else if(spi_ready_for_next_byte_reg)
		 begin
		     if(spi_ready_for_next_byte_counter < SPI_READY_COUNT)
			     spi_ready_for_next_byte_counter <= spi_ready_for_next_byte_counter + 1'd1;
			 else
			 begin
			     spi_ready_for_next_byte_counter <= 0;
	             spi_ready_for_next_byte <= 1'b0;
	             spi_ready_for_next_byte_reg <= 1'b0;
			 end
		 end
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //-----------------------------------------------
   // Register the SPI_CLK input with spi_clk_reg. 
   //-----------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	 spi_clk_reg <= 1'b0;
	else
	  begin
	     if ( spi_clk )
	       spi_clk_reg <= 1'b1;
	     else 
	       spi_clk_reg <= 1'b0;
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //------------------------------------------------
   // Input bit counter, maintains the count of serial
   // bits shifted in on SPI_MOSI
   //------------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	begin
	  in_bit_count <= 0;
	  bit_count_state <= BIT_COUNT_IDLE;
	end
	else
	  begin
	  
	     case( bit_count_state )
		     BIT_COUNT_IDLE:
			     if ( state_in[RECEIVE_DATA] )
				     bit_count_state <= BIT_COUNT_CLK_HI;
				 else if(state_in[IDLE])
				     in_bit_count <= 0;
			 BIT_COUNT_CLK_HI:
			     if( spi_clk_reg )
				 begin
				     bit_count_state <= BIT_COUNT_CLK_LOW;
	                 in_bit_count <= in_bit_count + 1;
				 end
			 BIT_COUNT_CLK_LOW:
			     if( !spi_clk_reg )
				     bit_count_state <= BIT_COUNT_CLK_HI;
				 else if( in_bit_count >= 8'h8)
				     bit_count_state <= BIT_COUNT_INC_BYTE;
			 BIT_COUNT_INC_BYTE:
			     if( state_in[RECEIVE_COMPLETE] | state_in[IDLE])
				 begin
				     bit_count_state <= BIT_COUNT_IDLE;
	                 in_bit_count <= 0;
				 end
			 default:
			 begin
				     bit_count_state <= BIT_COUNT_IDLE;
			 end
		 endcase
	  end
     end

   //------------------------------------------------
   // SPI input shift register
   //------------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	begin
			shift_in_reg <= 0;
			read_spi_data_in_once_flag <= 1'b1;
	end
	else
	  begin
	    if ( state_in[RECEIVE_DATA] & spi_clk_reg & read_spi_data_in_once_flag )
	    begin
		  shift_in_reg[0] <= spi_data_in;
		  shift_in_reg[7:1] <= shift_in_reg[6:0];
		  read_spi_data_in_once_flag <= 1'b0;
	    end
		else if ( state_in[RECEIVE_DATA] & !spi_clk_reg & !read_spi_data_in_once_flag )
		begin
          read_spi_data_in_once_flag <= 1'b1;
		end
	     else if (state_in[IDLE])
		 begin
	        shift_in_reg <= 0;
			read_spi_data_in_once_flag <= 1'b1;
	     end
	  end
     end // always @ (posedge CLK_66 or negedge RST)
 
   //------------------------------------------------
   // Output bit counter keeps track of which bit is 
   // being trasnmitted
   //------------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
      begin
	  out_bit_count <= 0;
        write_spi_data_out_once_flag <= 1'b1;
	end
	else
	  begin
	     if ( state_out[WRITE_DATA_BYTE] & !spi_clk_reg & write_spi_data_out_once_flag )
	     begin
	       	 out_bit_count <= out_bit_count + 1'd1;
			 write_spi_data_out_once_flag <= 1'b0; 
	     end
		 else if( state_out[WRITE_DATA_BYTE] & spi_clk_reg & !write_spi_data_out_once_flag )
 		 begin
        	 write_spi_data_out_once_flag <= 1'b1;
		 end
	     else if ( state_out[WRITE_DATA_COMPLETE] )
	         out_bit_count <= 0;
	  end
     end // always @ (posedge CLK_66 or negedge RST)

   //------------------------------------------------
   // SPI Shift Data Response out
   //------------------------------------------------
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     shift_out_reg <= 8'h30;
	     spi_data_out <= 1'b1;
		 write_spi_data_out_response_once <= 1'b1;
	  end
	else
	  begin

	     if (state_out[LOAD_SHIFT_REG] )
	     begin
	  		spi_data_out <= 0;
			shift_out_reg <= store_spi_output_byte;
	     end
	     else if (state_out[RECEIVE_DATA])
	     begin
		     spi_data_out <= spi_data_out;
		     shift_out_reg <= 0;
	     end
	     else if (state_out[IDLE] )
	     begin
	  		spi_data_out <= 0;
	     end
	     else if ( state_out[WRITE_DATA_BYTE] )
	     begin
			if( spi_clk_reg  & write_spi_data_out_response_once )
			begin
		  		spi_data_out <= shift_out_reg[7];
		  		shift_out_reg[7:1] <= shift_out_reg[6:0];
		 		shift_out_reg[0] <= 0;
				write_spi_data_out_response_once <= 0;
			end
			else if( !spi_clk_reg  & !write_spi_data_out_response_once )
			begin
				write_spi_data_out_response_once <= 1;
			end
	     end 
	  end
     end   
   
	  
   //--------------------------------------------------
   // SPI Incoming Data State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state_in <= 4'h00;
	     state_in[IDLE] <= 1'b1;
	  end
	else
	  state_in <= next_in;
     end

   always @ ( spi_data_in or in_bit_count or state_in or transfer_busy 
		 or spi_clk_reg or store_spi_output_byte or bit_count_state
		 )
     begin
	next_in = 4'h00;

	if (state_in[IDLE])
	begin
        if ( spi_clk_reg )
			next_in[RECEIVE_DATA] = 1'b1;
		else
			next_in[IDLE] = 1'b1;
	end

	if (state_in[RECEIVE_DATA])
	  begin
	     if ((in_bit_count >= 8) & spi_clk_reg)
	       next_in[SEND_TO_HOST] = 1'b1;
	     else
	       next_in[RECEIVE_DATA] = 1'b1;
	  end

	if (state_in[SEND_TO_HOST]) 
	  begin
	     if (!transfer_busy)
			  next_in[RECEIVE_COMPLETE] = 1'b1;
	     else
	         next_in[SEND_TO_HOST] = 1'b1;
	  end 	     

	  if (state_in[RECEIVE_COMPLETE]) 
	  begin
	     if (!spi_clk_reg)
			  next_in[IDLE] = 1'b1;
	     else
	         next_in[RECEIVE_COMPLETE] = 1'b1;
	  end 	     
	
`ifdef SIM
	if (state_in == (1 << IDLE))
	  state_name_in = "IDLE";
	else if (state_in == (1 << RECEIVE_DATA))
	  state_name_in = "RECEIVE_DATA";
	else if (state_in == (1 << SEND_TO_HOST))
	  state_name_in = "SEND_TO_HOST";
	else if (state_in == (1 << RECEIVE_COMPLETE))
	  state_name_in = "RECEIVE_COMPLETE";
`endif
     end // always @ (...
	

   //--------------------------------------------------
   // SPI Outgoing Data State Machine
   //--------------------------------------------------  
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state_out <= 6'h00;
	     state_out[IDLE] <= 1'b1;
	  end
	else
	  state_out <= next_out;
     end

   always @ ( spi_clk or spi_data_in or transfer_in_received
	     or in_bit_count or out_bit_count or state_out or transfer_busy 
		 or spi_clk_reg or store_spi_output_byte or bit_count_state
		 )
     begin
	next_out = 6'h00;

	if (state_out[IDLE])
	begin
        if ( spi_clk_reg )
			next_out[WRITE_REGISTER] = 1'b1;
		else
			next_out[IDLE] = 1'b1;
	end

	  if (state_out[WRITE_REGISTER])
	  begin
	       next_out[LOAD_SHIFT_REG] = 1'b1;
	  end

	if (state_out[LOAD_SHIFT_REG])
	       next_out[DELAY_WRITE_DATA_BYTE] = 1'b1;

	if (state_out[DELAY_WRITE_DATA_BYTE])
	  begin
	       next_out[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state_out[WRITE_DATA_BYTE])
	  begin
	     if (( out_bit_count >= 8 ))//& spi_clk_reg )
	       next_out[WRITE_DATA_COMPLETE] = 1'b1;
	     else
	       next_out[WRITE_DATA_BYTE] = 1'b1;
	  end

	if (state_out[WRITE_DATA_COMPLETE])
	  begin
	     if ( !spi_clk_reg )
	       next_out[IDLE] = 1'b1;
	     else
	       next_out[WRITE_DATA_COMPLETE] = 1'b1;
	  end

	
`ifdef SIM
	if (state_out == (1 << IDLE))
	  state_name_out = "IDLE";
	else if (state_out == (1 << WRITE_REGISTER))
	  state_name_out = "WRITE_REGISTER";
	else if (state_out == (1 << LOAD_SHIFT_REG))
	  state_name_out = "LOAD_SHIFT_REG";
	else if (state_out == (1 << DELAY_WRITE_DATA_BYTE))
	  state_name_out = "DELAY_WRITE_DATA_BYTE";
	else if (state_out == (1 << WRITE_DATA_BYTE))
	  state_name_out = "WRITE_DATA_BYTE";
	else if (state_out == (1 << WRITE_DATA_COMPLETE))
	  state_name_out = "WRITE_DATA_COMPLETE";
`endif
     end // always @ (...
	
   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
  active_control_register      ACTIVE_CONTROL_REG_INST
  (
   .CLK                        (CLK_66),
   .RST                        (RST),
   .TRANSFER_IN_RECEIVED       (transfer_in_received),
   .TRANSFER_IN_BYTE           (transfer_in_byte),

   .CONTROL_REGISTER           (control_register)
   );
   

	//-----------------------------------------------
   // Instantiate the EPT Active Transfer Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Active Modules
   //-----------------------------------------------
wire [22*2-1:0]  uc_out_m;
eptWireOR # (.N(2)) wireOR (UC_OUT, uc_out_m);
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);

	active_transfer            ACTIVE_TRANSFER_INST
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (start_transfer),
	 .transfer_received        (transfer_in_received),
	 
	 .transfer_busy            (transfer_busy),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	
endmodule
