/*******************************************************************************
** Filename:    TI2430Interface.c
**
** Created:     April 29 2013
**
** Author:      Richard Jolly
**
** Revision History
**         Date         Author  Comment
**      2013_04_29      WCH     Initial Version
**
*******************************************************************************/

/*******************************************************************************
**
** Include Files
**
*******************************************************************************/
#include <string.h>
#include "defines.h"
#include "typedefs.h"
#include "msp430x16x.h"
#include "prototype.h"
//#include "ZFMcommon.h"
#include "ZIGBpublic.h"
#include "ZIGBprivate.h"
#include "TI2430Interface.h"
#include "TI2430InterfacePrivate.h"
#include "ZIG_B_Flash_Memory_Tester.h"

/*******************************************************************************
**
**
**   SPI Bus initialization
**
**  
**
*******************************************************************************/


void vSPI_INIT(void)
{
  volatile unsigned int i;
  
  
  WDTCTL = WDTPW + WDTHOLD;                 // Stop watchdog
  P5SEL |= 0x0E;                            // P5.1,2,3 SPI option select. include 0 for CS?
  
  P5DIR |= 0x0A;
  P5DIR &= ~0x04;                           //5.1, 5.3 Output. 5.2 Input
 
  U1CTL = CHAR + SYNC + MM + SWRST;         // 8-bit, SPI, Master
  U1TCTL = SSEL1 + STC;                     // SMCLK, 3-wire; need CKPL?
  U1BR0 = 0x02;                             // SPICLK = SMCLK/2
  U1BR1 = 0x00;
  U1MCTL = 0x00;
  ME2 |= USPIE1;                            // Module enable
  U1CTL &= ~SWRST;                          // SPI enable
  for (i = 0x8FF; i > 0; i--);            // Time for slave to ready
}



/*******************************************************************************
** 4
** Name:  vTI2430DriverIOinit
**
** Parameters:  NONE
**
** Return:      VOID
**
** Outputs:
**
** Description:
**      Initializes GPIO for TI 2430
**
*******************************************************************************/

void vTI2430DriverIOinit( void )
{

  vSPI_INIT();
  P1SEL &= 0x00;
    
  P1DIR |= ~(0x00);
  P1DIR &= ~(0x78);
                            
  P1IE |= 0x20;
  P1IES &= ~0x20;
  P1IFG &= 0x00;
  
}

/*******************************************************************************
** 3
** Name:  vTI2430DeviceInit
**
** Parameters:  NONE
**
** Return:      VOID
**
** Outputs:     NONE
**
** Description:
**      Initialize TI2430
**
*******************************************************************************/

void vTI2430DeviceInit( void )
{

  ZFM_TI2430_RFCS_HI;
  ZFM_TI2430_RFRESET_HI;
  
  
  ZFM_TI2430_RFRESET_LO;
  ZFM_TI2430_RFRESET_HI;
  
}


/*******************************************************************************
**
** Name:  vTI2430DeviceSend
**
** Parameters:
**      pbyBuffer       Pointer to buffer to send
**      uiTxLength      Length of buffer
**
** Return:      VOID
**
** Outputs:
**
** Description:
**      Send buffer out the SPI0
**
*******************************************************************************/
void vTI2430DeviceSendReceive( ZFM_U08 *pbyBuffer, ZFM_U32 uiReadStart, ZFM_U32 uiTxLength )
{
  ZFM_U32      uiFlags;
  
  
  //
  //    Clear RF Ready Flag
  //
/* Add instruction to clear the RF_RDY interrupt */
    
  //
  //    Chip Select Low
  //    TelD-93 The chip select must be active for at least 750us before the first byte is transmitted by the master.
  //
  ZFM_TI2430_RFCS_LO;
  ZFM_TI2430_DELAY_USEC(1000);
  
  //
  // Send data if there is data to send
  //
  for ( ZFM_U32 uiLoop=0; uiLoop<uiTxLength; uiLoop++ )
  {
    ZFM_U08      byDataIn;
    ZFM_U32      uiResult;

    //  Tel-D Communications:
    //  1) Send Byte
    //  2) Wait for RF RDY interrupt or timeout
    //  3) Read character out of receive FIFO.
    //
   ZigBDriver(pbyBuffer[uiLoop]);


    //
    //  Do not overwrite outgoing TelD command.  Start storing after command info.
    //  Data still needs to be read but only stored after ReadStart index.
    //
    byDataIn = cSPI_READ();
    if ( uiLoop >= uiReadStart )
    {
      pbyBuffer[uiLoop] = byDataIn;
    }
    
    if ( uiLoop == (uiReadStart + 2) )
    {
      uiTxLength = uiLoop + byDataIn + 1;
    }
  }

  //
  //    Chip Select High
  //    TelD-94 The minimum time from the last byte sent to the chip-select being inactive must be 50us minimum.
  //
  ZFM_TI2430_DELAY_USEC(100);
  ZFM_TI2430_RFCS_HI;
  
} //    vTI2430DeviceSend


/*******************************************************************************
** 6
** Name:  ZigBDriver
**
** Parameters:  
**      *SendByte           char
**
** Return:      Indicates successful Command Transmission
**
** Outputs:
**
** Description:
**      State Machine which Sends byte out SPI Driver through U1TXBUF
**
*******************************************************************************/

char ZigBDriver(char SendByte)
{
       ZFM_U32 ZigBState = 0x00;
       char done = 0;
       
       while(!done)
       {
            switch (ZigBState)
            {
	       case 0x00:
	          vSPI_TRANSMIT (SendByte);
	          ZigBState = 0x01;
	       break;
	       case 0x01:
	          if(vTI2430DriverRFRDYRisingEdge())
	             ZigBState = 0x02;
                  else
                     return(0);
               break;
	       case 0x02:
	          if(vTI2430DriverRFRDYFallingEdge())
	             ZigBState = 0x03;
                  else
                     return(0);
               break;
	       case 0x03:
		  done = 0x01;
	          ZigBState = 0x00;
               break;
	       default:
               break;
	    }
       }

       return (1);

}

/*******************************************************************************
** 6
** Name:        vTI2430DriverRFRDYRisingEdge(void);
**
** Parameters:  NONE
**
**
** Return:      
**              NONE
**
** Outputs:
**
** Description:
**      Indicates RFNB interrupt rising edge
**
*******************************************************************************/

char vTI2430DriverRFRDYRisingEdge()
{

  unsigned int timercount = 0;
  while(!ZFM_RFRDY)
  {
    if(timercount++ > TIMEOUT_COUNT)
      return(0);
  }
  return (1);
}


/*******************************************************************************
** 6
** Name:        vTI2430DriverRFRDYfallingEdge(void);
**
** Parameters:  NONE
**
**
** Return:      
**              NONE
**
** Outputs:
**
** Description:
**      Indicates RFNB interrupt falling edge
**
*******************************************************************************/

char vTI2430DriverRFRDYFallingEdge( void )
{
  unsigned int timercount = 0;
    while (ZFM_RFRDY)
    {
     if (timercount++ > TIMEOUT_COUNT)
     return (0);
    }
  return (1);
}


/*******************************************************************************
** 6
** Name:  vSPI_TRANSMIT
**
** Parameters:  
**      transbyte                MOSI byte
**
** Return:      VOID
**
** Outputs:
**
** Description:
**      Sends byte out SPI Driver through U1TXBUF
**
*******************************************************************************/

void vSPI_TRANSMIT(char transbyte)
{  
  U1TXBUF = transbyte; 
}


/*******************************************************************************
** 6
** Name:  cSPI_READ
**
** Parameters:  NONE
**
**
** Return:      
**      receivebyte               MISO byte
**
** Outputs:
**
** Description:
**      Returns byte being read by SPI Driver through U1RXBUF
**
*******************************************************************************/

char cSPI_READ (void)
{
  char receivebyte;
  receivebyte = U1RXBUF;
  return (receivebyte);
}


/*******************************************************************************
** 14
** Name:  
**
** Parameters:  NONE
**
** Return:      VOID
**
** Outputs:
**
** Description:
**      RFINT ISR
**
*******************************************************************************/

#pragma vector=PORT1_VECTOR

 __interrupt void Port1ISR(void) 
 {
  if((P1IFG & 0x20) & P1IN)	
   {
    fZigBReceive = 0x01;
   }
  
  int_flags = P1IFG;
  P1IFG &= 0x00;
 }
